set nocompatible              " be iMproved, required
filetype off                  " required

" Section Plugins {{{
" Load vim-plug
if empty(glob("~/.config/nvim/autoload/plug.vim"))
    execute '!curl -fLo ~/.config/nvim/autoload/plug.vim --create-dirs https://raw.githubusercontent.com/junegunn/vim-plug/master/plug.vim'
endif
call plug#begin()

Plug 'tpope/vim-fugitive' " Git integration
Plug 'tpope/vim-rhubarb' " Github integration
Plug 'tpope/vim-sensible' " Defaults everyone can agree on
Plug 'tpope/vim-surround' " better handling of surrounding markers
Plug 'tpope/vim-ragtag' " Enhancement of surround for html
Plug 'tpope/vim-repeat' " repeat last command with .
Plug 'scrooloose/nerdtree', { 'on': ['NERDTreeToggle', 'NERDTreeFind'] } " file tree
Plug 'Xuyuanp/nerdtree-git-plugin', { 'on': ['NERDTreeToggle', 'NERDTreeFind'] }
Plug 'scrooloose/nerdcommenter' " better commenting
Plug 'neoclide/coc.nvim', {'branch': 'release'} " coc - completion
Plug 'easymotion/vim-easymotion' " better motion with leader
Plug 'bling/vim-airline' " statusline
Plug 'w0rp/ale' " asynchronous linting engine
Plug 'godlygeek/tabular' " Align text on symbols
Plug 'junegunn/fzf', { 'dir': '~/.fzf', 'do': './install --all' } " fuzzy file finder
Plug 'junegunn/fzf.vim' " vim integration for fzf
Plug 'airblade/vim-gitgutter' " git status in gutter column
Plug 'lambdalisue/suda.vim' " As w!! did not work anymore in nvim
Plug 'https://github.com/alok/notational-fzf-vim' " note taking
Plug 'prettier/vim-prettier', { 'do': 'npm install' } " prettier integration
Plug 'honza/vim-snippets' " Collection of Snippets, used by coc-snippet
"Plug 'ludovicchabant/vim-gutentags', { 'for': ['js', 'vue', 'php', 'python'] } " Keep tags in sync
Plug 'dpelle/vim-LanguageTool', { 'for': ['markdown', 'tex', 'plaintext', 'mail']} " LanguageTool bindings
Plug 'nvim-treesitter/nvim-treesitter', {'do': ':TSUpdate'}  " We recommend updating the parsers on update
Plug 'nvim-treesitter/nvim-treesitter-textobjects'
Plug 'kyazdani42/nvim-web-devicons' " Icons...
Plug 'mfussenegger/nvim-dap', { 'for': ['typescript', 'javascript', 'python', 'rust'] }
Plug 'lbrayner/vim-rzip' " recursive zip support, needed for yarn pnp
Plug 'kkoomen/vim-doge', { 'do': { -> doge#install() } }
Plug 'taybart/b64.nvim' " en/decode base64 strings

" telescope. Needed for octo.nvim, might replace fzf
Plug 'nvim-lua/popup.nvim'
Plug 'nvim-lua/plenary.nvim'
Plug 'nvim-telescope/telescope.nvim'
Plug 'nvim-telescope/telescope-fzf-native.nvim', { 'do': 'make' } " improve sort performance
Plug 'pwntester/octo.nvim'

"Plug 'github/copilot.vim'

" {{{ CocPlugins
Plug 'neoclide/coc-json', { 'do': 'yarn install --frozen-lockfile' }
Plug 'neoclide/coc-snippets', { 'do': 'yarn install --frozen-lockfile' }
Plug 'neoclide/coc-tsserver', { 'do': 'yarn install --frozen-lockfile' }
Plug 'neoclide/coc-eslint', { 'do': 'yarn install --frozen-lockfile' }
Plug 'fannheyward/coc-rust-analyzer', { 'do': 'yarn install --frozen-lockfile' }
Plug 'neoclide/coc-css', { 'do': 'yarn install --frozen-lockfile' }
Plug 'neoclide/coc-vimtex', { 'do': 'yarn install --frozen-lockfile' }
Plug 'fannheyward/coc-texlab', { 'do': 'yarn install --frozen-lockfile' }
Plug 'neoclide/coc-html', { 'do': 'yarn install --frozen-lockfile' }
Plug 'neoclide/coc-yaml', { 'do': 'yarn install --frozen-lockfile' }
"Plug 'neoclide/coc-python', { 'do': 'yarn install --frozen-lockfile' }
Plug 'fannheyward/coc-sql', { 'do': 'yarn install --frozen-lockfile' }
Plug 'fannheyward/coc-markdownlint', { 'do': 'yarn install --frozen-lockfile' }
Plug 'fannheyward/coc-pyright', { 'for': 'python', 'do': 'yarn install --frozen-lockfile' }
Plug 'josa42/coc-sh', { 'do': 'yarn install --frozen-lockfile' }
" }}}

" html / templates{{{
Plug 'mattn/emmet-vim', { 'for': ['jinja.html', 'html', 'php', 'gohtmltmpl', 'vue', 'markdown'] } " emmet support for vim - easily create markdup wth CSS-like syntax
Plug 'gregsexton/MatchTag', { 'for': ['html', 'php', 'gohtmltmpl', 'vue'] } " match tags in html, similar to paren support
Plug 'othree/html5.vim', { 'for': ['html', 'php', 'gohtmltmpl', 'vue'] } " html5 support
Plug 'lumiliet/vim-twig', { 'for': ['html'] } " twig syntax support
Plug 'Glench/Vim-Jinja2-Syntax', { 'for': ['html'] }
Plug 'jvanja/vim-bootstrap4-snippets', { 'for': ['html', 'blade'] }
"}}}

" JavaScript{{{
Plug 'gavocanov/vim-js-indent', { 'for': [ 'javascript' ]} " JavaScript indent support
Plug 'posva/vim-vue', { 'for': [ 'javascript', 'html', 'vue' ] } " vue.js integration
Plug 'kevinoid/vim-jsonc'
"}}}

" LaTeX{{{
Plug 'lervag/vimtex', { 'for': 'tex' }
"}}}

" Markdown{{{
Plug 'tpope/vim-markdown', { 'for': 'markdown'}
Plug 'iamcco/markdown-preview.nvim', { 'do': 'cd app & yarn install', 'for': 'markdown'  }
"}}}

" Colorschemes{{{
Plug 'altercation/vim-colors-solarized'
Plug 'vim-scripts/nginx.vim'
"}}}

" TOML{{{
Plug 'cespare/vim-toml', { 'for': 'toml' }
"}}}

" C/C++{{{
Plug 'Rip-Rip/clang_complete', { 'for': ['c', 'cpp'], 'do': 'make install' }
let g:clang_library_path='/usr/lib/llvm-6.0/lib/libclang-6.0.so.1'
Plug 'rhysd/vim-clang-format', { 'for' : ['c', 'cpp', 'java'] }
Plug 'vim-scripts/DoxygenToolkit.vim', { 'for' : ['c', 'cpp'] }
"}}}

" PHP{{{
Plug 'captbaritone/better-indent-support-for-php-with-html', { 'for': 'php' }
Plug 'lumiliet/vim-twig', { 'for': [ 'php', 'html' ] } " Twig templates
Plug 'Rican7/php-doc-modded', { 'for': 'php' } " Automatic phpdoc comments
Plug 'jwalton512/vim-blade', { 'for': 'php' }
"}}}

" {{{ Python
Plug 'jeetsukumaran/vim-pythonsense', { 'for': ['python'] } " text objects for python
Plug 'heavenshell/vim-pydocstring', { 'for': ['python'], 'do': 'make install' }
Plug 'anntzer/vim-cython', { 'for': ['python']} 
Plug 'python-rope/ropevim'
"}}}

"{{{ CSV
Plug 'chrisbra/csv.vim', { 'for': 'csv' }
"}}}

" All of your Plugins must be added before the following line
call plug#end()            " required
filetype plugin indent on    " required
" }}}

" THEME{{{
colorscheme solarized
set background=dark

" Use light theme if whole shell is in light mode
let themeenv=$THEME_SWITCHER
if themeenv=='light'
    set background=light
endif
" }}}

" SETTINGS{{{
" disable Ex mode, who needs this anyway?
noremap Q <Nop>
set number                   " line numbers
set autoread                 " reload file automatically
set colorcolumn=100          " highlight column 99
set textwidth=99
set smartcase "use smart case like in ripgrep (case insensitive until capital letters are used)

set history=1000
set wildignore+=node_modules " ignore node_modules folder from file search

set inccommand=split         " show interactive results of substiute command

" make comments and HTML attributes italic
highlight Comment cterm=italic
highlight htmlArg cterm=italic

set autoindent               " automatically set indent of new line
set smartindent
set tabstop=4
set shiftwidth=4
set expandtab

" toggle invisible characters
set list
set listchars=tab:→\ ,trail:⋅,extends:❯,precedes:❮
set showbreak=↪

set ttyfast                 " faster redrawing
set diffopt+=vertical
set laststatus=2            " show the satus line all the time
set so=7                    " set 7 lines to the cursors - when moving vertical
set wildmenu                " enhanced command line completion
set hidden                  " current buffer can be put into background
set showcmd                 " show incomplete commands
" set noshowmode              " don't show which mode disabled for PowerLine
set wildmode=list:longest   " complete files like a shell
set scrolloff=3             " lines of text around cursor
set shell=$SHELL
set cmdheight=1             " command bar height
set title                   " set terminal title
set mouse=nv

set foldmethod=marker"      " folds are marked with {{{}}}

" Uncomment the following to have Vim jump to the last position when
" reopening a file
if has("autocmd")
    if expand('%:t') != 'COMMIT_EDITMSG'
        au BufReadPost * if line("'\"") > 0 && line("'\"") <= line("$")
                    \| exe "normal! g'\"" | endif
    endif
endif

" Setting for correct mips syntax highlighting
au BufReadPost *.mips set syntax=mips

" Override default flavor to latex, so vimtex gets loaded always
let g:tex_flavor = "latex"

" use python 3 when in doubt
let g:python3_host_prog="/home/schneider/.pyenv/versions/neovim3/bin/python"
set pyxversion=3

" nodejs settings
"let g:node_host_prog="/usr/bin/node"

" Disable ALEs LSP, as we're using coc
let g:ale_disable_lsp = 1
" }}}

" MAPPINGS{{{
let mapleader = ","
" CtrlP for Tags
nnoremap <leader><C-p> :CtrlPTag<cr>

" fzf bindings
"nmap <C-p> :Files<CR>
nmap <leader><C-p> :Tags<CR>
nmap <leader>/ :Rg<CR>
"nmap <leader>b :Buffers<CR>

" Telescope Bindings
nmap <C-p> :lua require("telescope.builtin").find_files({find_command={"fd","--type=f","-H", "-E=.git", "-E=.yalc", "-E=node_modules"}})<CR>
"nmap <C-p> :Telescope fd<CR>
nmap <leader>b :lua require('telescope.builtin').buffers({ sort_mru = true, ignore_current_buffer = true })<CR>
nmap <leader>/ :Telescope live_grep<CR>


" Toggle NERDTree
nmap <silent> <leader>k :NERDTreeToggle<cr>
" expand to the path of the file in the current buffer
nmap <silent> <leader>y :NERDTreeFind<cr>
" Allow saving of files as sudo when I forgot to start vim using sudo.
cmap w!! :w suda://%
"Toggle set paste
nmap <leader>tp :set paste!<cr>
" scroll the viewport faster
nnoremap <C-e> 3<C-e>
nnoremap <C-y> 3<C-y>
" ctrl-r in visual mode to replace selected text
vnoremap <C-r> "hy:%s/<C-r>h//g<left><left>

nmap <leader>an :ALENext<cr>

nnoremap <leader>tn :tabnew<cr>

nnoremap <F3> :Git<cr>
nnoremap <F4> :Git push<cr>

vnoremap <silent> <leader>be :<c-u>lua require("b64").encode()<cr>
vnoremap <silent> <leader>bd :<c-u>lua require("b64").decode()<cr>
"}}}

" {{{ Textobjects
lua <<EOF
require'nvim-treesitter.configs'.setup {
  textobjects = {
    select = {
      enable = true,

      -- Automatically jump forward to textobj, similar to targets.vim 
      lookahead = true,

      keymaps = {
        -- You can use the capture groups defined in textobjects.scm
        ["af"] = "@function.outer",
        ["if"] = "@function.inner",
        ["ac"] = "@class.outer",
        ["ic"] = "@class.inner",

        -- Or you can define your own textobjects like this
        -- ["iF"] = {
        --   python = "(function_definition) @function",
        --   cpp = "(function_definition) @function",
        --   c = "(function_definition) @function",
        --   java = "(method_declaration) @function",
        -- },
      },
    },
  },
}
EOF
" }}}

" P L U G I N S{{{
let g:languagetool_server="/opt/LanguageTool/LanguageTool-5.0/languagetool-server.jar"
let g:languagetool_jar="/opt/LanguageTool/LanguageTool-5.0/languagetool-commandline.jar"

let NERDTreeShowHidden=1

if exists('g:started_by_firenvim')
    let g:airline_disable_statusline = 1
else
    let g:airline_powerline_fonts = 1
end

" notational notes
let g:nv_search_paths = ['~/notes']
let g:nv_default_extension = '.md'
let g:nv_main_directory = '~/notes'

" Octo.nvim
"lua require('octo')

" ALE {{{
let g:airline#extensions#ale#enabled = 1
"}}}

let vim_tags_ctags_binary='~/bin/phpctags'"

" Java config
autocmd FileType java setlocal omnifunc=javacomplete#Complete


" Vimtex{{{
if !exists('g:neocomplete#sources#omni#input_patterns')
	let g:neocomplete#sources#omni#input_patterns = {}
endif
let g:neocomplete#sources#omni#input_patterns.tex =
			\ '\v\\%('
			\ . '\a*%(ref|cite)\a*%(\s*\[[^]]*\])?\s*\{[^{}]*'
			\ . '|includegraphics%(\s*\[[^]]*\])?\s*\{[^{}]*'
			\ . '|%(include|input)\s*\{[^{}]*'
			\ . ')'
"}}}

" C/C++ {{{
autocmd FileType c,cpp,objc nnoremap <buffer><Leader>f :<C-u>ClangFormat<CR>
autocmd FileType c,cpp,objc vnoremap <buffer><Leader>f :ClangFormat<CR>
"}}}

" Prettier
let g:prettier#config#tab_width = 4
let g:prettier#exec_cmd_async = 1

" Completion {{{

" if hidden is not set, TextEdit might fail.
set hidden

" Some servers have issues with backup files, see #649
set nobackup
set nowritebackup

" Better display for messages
set cmdheight=2

" You will have bad experience for diagnostic messages when it's default 4000.
set updatetime=300

" don't give |ins-completion-menu| messages.
set shortmess+=c

" always show signcolumns
set signcolumn=yes

" {{{ coc completion
"Use <tab> and <S-tab> to navigate completion list:  

function! CheckBackSpace() abort
    let col = col('.') - 1
    return !col || getline('.')[col - 1]  =~ '\s'
endfunction
inoremap <silent><expr> <CR> coc#pum#visible() ? coc#_select_confirm()
            \: "\<C-g>u\<CR>\<c-r>=coc#on_enter()\<CR>"

" Insert <tab> when previous text is space, refresh completion if not.
inoremap <silent><expr> <TAB>
            \ coc#pum#visible() ? coc#pum#next(1):
            \ CheckBackSpace() ? "\<Tab>" :
            \ coc#refresh()
inoremap <expr><S-TAB> coc#pum#visible() ? coc#pum#prev(1) : "\<C-h>"
"}}}

function! s:check_back_space() abort
  let col = col('.') - 1
  return !col || getline('.')[col - 1]  =~# '\s'
endfunction

" Use <c-space> to trigger completion.
inoremap <silent><expr> <c-space> coc#refresh()

" Use `[c` and `]c` to navigate diagnostics
nmap <silent> [c <Plug>(coc-diagnostic-prev)
nmap <silent> ]c <Plug>(coc-diagnostic-next)

" Remap keys for gotos
nmap <silent> gd <Plug>(coc-definition)
nmap <silent> gy <Plug>(coc-type-definition)
nmap <silent> gi <Plug>(coc-implementation)
nmap <silent> gr <Plug>(coc-references)

" Use K to show documentation in preview window
nnoremap <silent> H :call <SID>show_documentation()<CR>

function! s:show_documentation()
  if (index(['vim','help'], &filetype) >= 0)
    execute 'h '.expand('<cword>')
  else
    call CocAction('doHover')
  endif
endfunction

" Highlight symbol under cursor on CursorHold
autocmd CursorHold * silent call CocActionAsync('highlight')

" Remap for rename current word
nmap <leader>rn <Plug>(coc-rename)

" Remap for format selected region
xmap <leader>f  <Plug>(coc-format-selected)
nmap <leader>f  <Plug>(coc-format-selected)

augroup mygroup
  autocmd!
  " Setup formatexpr specified filetype(s).
  autocmd FileType typescript,json setl formatexpr=CocAction('formatSelected')
  " Update signature help on jump placeholder
  autocmd User CocJumpPlaceholder call CocActionAsync('showSignatureHelp')
augroup end

" Remap for do codeAction of selected region, ex: `<leader>aap` for current paragraph
xmap <leader>a  <Plug>(coc-codeaction-selected)
nmap <leader>a  <Plug>(coc-codeaction-selected)

" Remap for do codeAction of current line
nmap <leader>ac  <Plug>(coc-codeaction)
" Fix autofix problem of current line
nmap <leader>qf  <Plug>(coc-fix-current)

" Use <tab> for select selections ranges, needs server support, like: coc-tsserver, coc-python
"nmap <silent> <TAB> <Plug>(coc-range-select)
"xmap <silent> <TAB> <Plug>(coc-range-select)
"xmap <silent> <S-TAB> <Plug>(coc-range-select-backward)

" Use `:Format` to format current buffer
command! -nargs=0 Format :call CocAction('format')

" Use `:Fold` to fold current buffer
command! -nargs=? Fold :call     CocAction('fold', <f-args>)

" use `:OR` for organize import of current buffer
command! -nargs=0 OR   :call     CocAction('runCommand', 'editor.action.organizeImport')

" Add status line support, for integration with other plugin, checkout `:h coc-status`
set statusline^=%{coc#status()}%{get(b:,'coc_current_function','')}

" Using CocList
" Show all diagnostics
nnoremap <silent> <space>a  :<C-u>CocList diagnostics<cr>
" Manage extensions
nnoremap <silent> <space>e  :<C-u>CocList extensions<cr>
" Show commands
nnoremap <silent> <space>c  :<C-u>CocList commands<cr>
" Find symbol of current document
nnoremap <silent> <space>o  :<C-u>CocList outline<cr>
" Search workspace symbols
nnoremap <silent> <space>s  :<C-u>CocList -I symbols<cr>
" Do default action for next item.
nnoremap <silent> <space>j  :<C-u>CocNext<CR>
" Do default action for previous item.
nnoremap <silent> <space>k  :<C-u>CocPrev<CR>
" Resume latest coc list
nnoremap <silent> <space>p  :<C-u>CocListResume<CR>:

" snippets
imap <C-k> <Plug>(coc-snippets-expand-jump)

let g:neosnippet#snippets_directory = '~/.dotfiles/vim/neosnippets'
"}}}

" Treesitter {{{
lua <<EOF
require'nvim-treesitter.configs'.setup {
  ensure_installed = {"typescript", "javascript", "python", "json", "yaml"}, -- one of "all", "maintained" (parsers with maintainers), or a list of languages
  highlight = {
    enable = true,              -- false will disable the whole extension
    disable = {},  -- list of language that will be disabled
  },
}
EOF
" }}}

" Telescope {{{
lua <<EOF
-- You dont need to set any of these options. These are the default ones. Only
-- the loading is important
require('telescope').setup {
  extensions = {
    fzf = {
      fuzzy = true,                    -- false will only do exact matching
      override_generic_sorter = true,  -- override the generic sorter
      override_file_sorter = true,     -- override the file sorter
      case_mode = "smart_case",        -- or "ignore_case" or "respect_case"
                                       -- the default case_mode is "smart_case"
    }
  }
}
-- To get fzf loaded and working with telescope, you need to call
-- load_extension, somewhere after setup function:
require('telescope').load_extension('fzf')
EOF
"}}}

" doge settings
let g:doge_javascript_settings = {
\  'omit_redundant_param_types': 1,
\}
let g:doge_doc_standard_python = 'google'
let g:doge_mapping_comment_jump_forward = '<c-k>'

"}}}
